/*
 * Copyright (c) 2002, 2003 MATPOCKuH.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <unistd.h>
#include <libgen.h>
#include <sysexits.h>
#include <stdlib.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "settings.h"
#include "logger.h"
#include "acd.h"
#include "auto_cd.h"
#include "version.h"

const char *progname = "autocd";
void
usage(char *argv[])
{
    logger("%s\n\
Usage: %s [-hc] [-d device] [-p mountpoint] [-f fstype] [-o options]\n\
\t[-s socket] [-u user] [-g group] [-m mode]\n\
  -h\t\t- this help\n\
  -c\t\t- don't close devices while playing is in progress\n\
  -d device\t- device for monitoring by autocd\n\
  -p mountpoint\t- mount point for device\n\
  -f fstype\t- file system type for device\n\
  -o options\t- mount options for device\n\
  -s socket\t- path to a control socket\n\
  -u user\t- socket owner\n\
  -g group\t- socket group\n\
  -m mode\t- socket mode\n",
	ver_getversionstring(), basename(argv[0]));

    exit(EX_USAGE);
}

int
main(int argc, char *argv[])
{
    int ch, devices_selected = 0, devices_ok = 0, last_device = 0;

    autocd_init();

    while((ch = getopt(argc, argv, "hcd:p:f:o:s:u:g:m:")) != -1) {
	switch(ch) {
	case 'c':
	    autocd_noclose();
	    break;

	case 'd':
	    devices_selected = 1;
	    if(!autocd_initdevice(optarg))
		devices_ok = last_device = 1;
	    else
		last_device = 0;

	    break;

	case 'p':
	case 'f':
	case 'o':
	    if(devices_selected) {
		if(!last_device) {
		    logger("Last device is not usable. -%c ignored\n", ch);
		    break;
		}
	    } else {
		devices_selected = 1;
		if(autocd_initdevice(CDDEV))
		    break;

		devices_ok = 1;
	    }
		    
	    switch(ch) {
	    case 'p':
		autocd_set_mnt_point(optarg);
		break;

	    case 'f':
		autocd_set_mnt_fstype(optarg);
		break;

	    case 'o':
		autocd_set_mnt_opts(optarg);
		break;
	    }

	    break;

	case 's':
	    acd_setsocketpath(optarg);
	    break;

	case 'u':
	    if(autocd_setsocketowner(optarg))
		exit(EX_NOUSER);
	    break;

	case 'g':
	    if(autocd_setsocketgroup(optarg))
		exit(EX_NOUSER);
	    break;

	case 'm':
	    if(autocd_setsocketmode(optarg))
		exit(EX_USAGE);
	    break;

	case 'h':
	default:
		usage(argv);
	}
    }

    if(argc != optind)
	usage(argv);

    if(!devices_selected) {
	if(!autocd_initdevice(CDDEV))
	    devices_ok = 1;
    }

    if(!devices_ok) {
	logger("No usable devices. Exiting.\n");
        exit(EX_USAGE);
    }

    if(autocd_initsocket())
	exit(EX_NOPERM);

    if(autocd_daemon())
	exit(EX_OSERR);

    return 0;
}
