/*
 * Copyright (c) 2002, 2003 MATPOCKuH.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>

#include "mqueue.h"

mq_queue_t *
mq_makequeue(size_t recordsize)
{
    mq_queue_t *tmp;

    tmp = malloc(sizeof(mq_queue_t));

    tmp->head = tmp->tail = tmp->cur = tmp->next = NULL;
    tmp->rsize = recordsize;

    return tmp;
}

void
mq_addrecord(mq_queue_t *mq, mq_record_t *record)
{
    record->next = NULL;
    record->prev = mq->tail;

    if(mq->tail) {
	mq->tail->next = record;
	mq->tail = record;
    } else {
	mq->head = mq->tail = record;
    }
}

mq_record_t *
mq_addnewrecord(mq_queue_t *mq)
{
    mq_record_t *tmp;

    tmp = malloc(mq->rsize);

    mq_addrecord(mq, tmp);

    return tmp;
}

mq_record_t *
mq_resetcurrent(mq_queue_t *mq)
{
    return (mq->next = mq->head);
}

mq_record_t *
mq_getcurrent(mq_queue_t *mq)
{
    return mq->cur;
}

mq_record_t *
mq_gethead(mq_queue_t *mq)
{
    return mq->head;
}

mq_record_t *
mq_gettail(mq_queue_t *mq)
{
    return mq->tail;
}

mq_record_t *
mq_nextrecord(mq_queue_t *mq)
{
    mq->cur = mq->next;

    if(!mq->next)
	return NULL;

    mq->next = mq->next->next;

    return mq->cur;
}

mq_record_t *
mq_removecurrent(mq_queue_t *mq)
{
    mq_record_t *tmp = mq->cur;

    if(!tmp)
	return NULL;

    if(tmp->next)
	tmp->next->prev = tmp->prev;
    else
	mq->tail = tmp->prev;

    if(tmp->prev)
	tmp->prev->next = tmp->next;
    else
	mq->head = tmp->next;

    free(tmp);

    return mq->cur;
}
